#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pickle
import tkinter as tk
import os

from tkinter import filedialog, Label, Checkbutton, StringVar, font
from PIL import Image, ImageTk

"""
    This script constructs SHIRE's user interface'    
"""

class general_settings:

    def __init__(self, master):
        super().__init__()
        self.master = master
        self.settings_import = False
        self.row = 0
        self.master.winfo_toplevel().title(
            "SHIRE - Susceptibility and Hazard mappIng fRamEwork")
        
        default_font = font.nametofont("TkDefaultFont")
        heading_font = font.Font(family=default_font.cget("family"), size=14, weight="bold")
        
        impath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')) + '/images/Logo.png'
        pil_image = Image.open(impath)
        resized_image = pil_image.resize((450, 120), Image.LANCZOS)
        image = ImageTk.PhotoImage(resized_image)

        image_label = tk.Label(self.master, image=image, width=450, height=120)
        image_label.image = image
        image_label.grid(row=self.row, column=0, columnspan=2)
        self.row = self.row + 1

        self.dic_change = {}

        b_chooseFile = tk.Button(self.master, text="Import settings",
                                 command=self.import_file)
        b_chooseFile.grid(row=self.row, column=0, columnspan=4, sticky='ew')
        self.row = self.row + 1

        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="Intention of this run", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, sticky='w')
        self.train = tk.IntVar()
        self.row = self.row + 1
        Label(self.master, text="Generation of the...", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.train = tk.IntVar()

        b1 = Checkbutton(self.master, text="Training dataset",
                         variable=self.train)
        b1.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        self.pred = tk.IntVar()
        b2 = Checkbutton(self.master, text="Prediction dataset",
                         variable=self.pred)
        b2.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        self.map = tk.IntVar()
        b3 = Checkbutton(self.master, text="Susceptibility or hazard map",
                         variable=self.map)
        b3.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="General settings", anchor='w', justify='left', font=heading_font).grid(row=self.row, column=0, sticky='w')
        self.row = self.row + 1
        Label(self.master, text="Resolution [m]", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.res = tk.IntVar()
        i = 'resolution'
        self.res.trace_add("write", lambda name, index, mode, var=self.res,
                            i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.res)
        entry.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1

        Label(self.master, text="No data value", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.no_value = tk.IntVar()
        i = 'no_value'
        self.no_value.trace_add("write", lambda name, index, mode,
                                var=self.no_value, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.no_value)
        entry.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="CRS", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.crs = tk.StringVar()
        i = 'crs'
        self.crs.trace_add("write", lambda name, index, mode, var=self.crs,
                           i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.crs)
        entry.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="Random seed", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.random_seed = tk.IntVar()
        i = 'random_seed'
        self.random_seed.trace_add(
            "write", lambda name, index, mode, var=self.random_seed,
            i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.random_seed)
        entry.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        self.save = tk.IntVar()
        b4 = Checkbutton(self.master, text="Save the settings for later use",
                         variable=self.save, anchor='w', justify='left', font=font.Font(family=default_font.cget("family"), weight="bold"))
        b4.grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        btn = tk.Button(self.master, text="Submit", command=self.on_submit)
        btn.grid(row=self.row, column=0, columnspan=4, sticky='ew')

    def import_file(self):
        sourceFile = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')

        with open(sourceFile, 'rb') as handle:
            self.properties = pickle.load(handle)

        self.settings_import = True

    def callback(self, var, i):
        self.dic_change[i] = var.get()

    def on_submit(self):

        if self.train.get() == 0:
            self.train = False
        else:
            self.train = True

        if self.pred.get() == 0:
            self.pred = False
        else:
            self.pred = True

        if self.map.get() == 0:
            self.map = False
        else:
            self.map = True

        if self.settings_import:
            self.res = int(self.properties['resolution'])
            self.no_value = int(self.properties['no_value'])
            self.crs = self.properties['crs']
            self.random_seed = int(self.properties['random_seed'])
        else:
            self.res = int(self.res.get())
            self.no_value = int(self.no_value.get())
            self.crs = self.crs.get()
            self.random_seed = self.random_seed.get()

        if not self.settings_import:

            dic = {}
            dic['resolution'] = self.res
            dic['random_seed'] = self.random_seed
            dic['crs'] = self.crs
            dic['no_value'] = self.no_value
            dic['train'] = self.train
            dic['pred'] = self.pred
            dic['map'] = self.map

        elif self.settings_import:

            dic = {}
            dic['resolution'] = self.properties['resolution']
            dic['random_seed'] = self.properties['random_seed']
            dic['crs'] = self.properties['crs']
            dic['no_value'] = self.properties['no_value']
            dic['train'] = self.train
            dic['pred'] = self.pred
            dic['map'] = self.map

            for key in self.dic_change:
                dic[key] = self.dic_change[key]

        with open('tmp_settings.pkl', 'wb') as handle:
            pickle.dump(dic, handle)

        if self.save.get() == 1:

            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings.pkl'

            with open(sourceFile, 'wb') as handle:
                pickle.dump(dic, handle)

        dic = {}
        dic['resolution'] = self.res
        dic['random_seed'] = self.random_seed
        dic['crs'] = self.crs
        dic['no_value'] = self.no_value
        dic['train'], dic['pred'], dic['map'] = self.train, self.pred, self.map

        self.master.destroy()

        if self.train:
            settings_train(self.pred, self.map)
        elif self.pred:
            settings_pred(self.map)
        elif self.map:
            settings_map()

class settings_pred:

    def __init__(self, maps):
        super().__init__()
        self.master = tk.Tk()
        self.map = maps
        self.settings_import = False
        default_font = font.nametofont("TkDefaultFont")
        heading_font = font.Font(family=default_font.cget("family"), size=14, weight="bold")
        self.row = 0

        self.master.winfo_toplevel().title(
            "Prediction dataset generation - general settings")
        
        impath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')) + '/images/Logo.png'
        pil_image = Image.open(impath)
        resized_image = pil_image.resize((600, 180), Image.LANCZOS)
        image = ImageTk.PhotoImage(resized_image)
        
        image_label = tk.Label(self.master, image=image, width=600, height=180)
        image_label.image = image
        image_label.grid(row=self.row, column=0, columnspan=2)
        self.row = self.row + 1
    
        b_chooseFile = tk.Button(
            self.master, text="Import settings", command=self.import_file)
        b_chooseFile.grid(row=self.row, column=0, columnspan=4, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="General information", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, sticky='w')
        self.row = self.row + 1

        Label(self.master, text="Path to summary on geospatial data", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')

        self.b_gs = tk.Button(self.master, text="Choose path to dataset_summary.csv",
                              command=lambda: self.choose_path('geo_path'))
        self.b_gs.grid(row=self.row, column=1, columnspan=1, sticky='ew')
        self.row = self.row + 1

        Label(self.master, text="Features to include", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')

        self.b_feat = tk.Button(self.master, text="Choose path to keys_to_include.csv",
                                command=lambda: self.choose_path('feat_path'))
        self.b_feat.grid(row=self.row, column=1, columnspan=1, sticky='ew')
        self.row = self.row + 1

        self.dic_path = {}
        self.dic_change = {}

        Label(self.master,
              text="Path to directory for storing\n the prediction dataset", anchor='w', justify='left').grid(
                  row=self.row, column=0, sticky='w')

        self.b_train = tk.Button(self.master,
                                 text="Choose directory",
                                 command=lambda: self.choose_dir('pred_path'))
        self.b_train.grid(row=self.row, column=1, columnspan=1, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
                
        self.ohe = tk.IntVar()
        b11 = Checkbutton(self.master, text="One-hot encoding of categorical variables",
                         variable=self.ohe, anchor='w', justify='left')#, command=lambda: button_pressed.set('cluster'))
        b11.grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="Area of interest", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, sticky='w')
        self.row = self.row + 1
        Label(self.master, text="Bounding box", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
            
        self.east = tk.StringVar()
        i = 'east'
        self.east.trace_add(
            "write", lambda name, index, mode,
            var=self.east, i=i: self.callback(var, i))
        self.entry1 = tk.Entry(self.master, textvariable=self.east, fg='grey')
        placeholder1 = 'East [decimal degrees]'
        self.entry1.insert(0, placeholder1)  # Insert placeholder text initially
        self.entry1.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        self.west = tk.StringVar()
        i = 'west'
        self.east.trace_add(
            "write", lambda name, index, mode,
            var=self.west, i=i: self.callback(var, i))
        self.entry2 = tk.Entry(self.master, textvariable=self.west, fg='grey')
        placeholder2 = 'West [decimal degrees]'
        self.entry2.insert(0, placeholder2)  # Insert placeholder text initially
        self.entry2.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        self.north = tk.StringVar()
        i = 'north'
        self.east.trace_add(
            "write", lambda name, index, mode,
            var=self.north, i=i: self.callback(var, i))
        self.entry3 = tk.Entry(self.master, textvariable=self.north, fg='grey')
        placeholder3 = 'North [decimal degrees]'
        self.entry3.insert(0, placeholder3)  # Insert placeholder text initially
        self.entry3.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1
        
        self.south = tk.StringVar()
        i = 'south'
        self.east.trace_add(
            "write", lambda name, index, mode,
            var=self.south, i=i: self.callback(var, i))
        self.entry4 = tk.Entry(self.master, textvariable=self.south, fg='grey')
        placeholder4 = 'South [decimal degrees]'
        self.entry4.insert(0, placeholder4)  # Insert placeholder text initially
        self.entry4.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.grid(row=self.row, column=1, sticky='ew')
        self.row = self.row + 1

        self.save = tk.IntVar()
        b4 = Checkbutton(self.master,
                         text="Save above settings for later use",
                         variable=self.save, anchor='w', justify='left', font=font.Font(family=default_font.cget("family"), weight="bold"))
        b4.grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="What do you want to do?", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, sticky='w')
        self.row = self.row + 1
        Label(self.master, text="Choose from:", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')

        button_pressed = StringVar()
        self.from_scratch = tk.IntVar()
        b1 = tk.Radiobutton(self.master,
                            text="Generate prediction dataset\n from scratch",
                            variable=button_pressed,
                            command=lambda: button_pressed.set('from_scratch'), anchor='w', justify='left')
        b1.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        
        self.delete = tk.IntVar()
        b2 = tk.Radiobutton(self.master,
                            text="Delete feature(s) from\n existing prediction dataset",
                            variable=button_pressed,
                            command=lambda: button_pressed.set('delete'), anchor='w', justify='left')
        b2.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        
        self.add = tk.IntVar()
        b3 = tk.Radiobutton(self.master,
                            text="Add feature(s) from\n existing prediction dataset",
                            variable=button_pressed,
                            command=lambda: button_pressed.set('add'), anchor='w', justify='left')
        b3.grid(row=self.row, column=1, sticky='w')
        self.row = self.row + 1
        
        all_placeholders = []
        all_placeholders.append(placeholder1)
        all_placeholders.append(placeholder2)
        all_placeholders.append(placeholder3)
        all_placeholders.append(placeholder4)

        b2.wait_variable(button_pressed)

        if button_pressed.get() == 'from_scratch':
            self.delete.set(0)
            self.add.set(0)
            self.from_scratch.set(1)
        elif button_pressed.get() == 'add':
            self.delete.set(0)
            self.add.set(1)
            self.from_scratch.set(0)
        elif button_pressed.get() == 'delete':
            self.delete.set(1)
            self.add.set(0)
            self.from_scratch.set(0)

        btn = tk.Button(self.master, text="Submit", command=lambda: self.submit(all_placeholders))
        btn.grid(row=self.row, column=0, columnspan=3, sticky='ew')
        
    def on_entry_click(self, event, entry, placeholder):
        """Function that handles the event when the entry field is clicked."""
        if entry.get() == placeholder:
            entry.delete(0, tk.END)  # Delete all the text in the entry
            entry.config(fg='black')  # Change the text color to black
    
    def on_focus_out(self, event, entry, placeholder):
        """Function that handles the event when the entry field loses focus."""
        if entry.get() == '':
            entry.insert(0, placeholder)  # Insert the placeholder text
            entry.config(fg='grey')  # Change the text color to grey

    def import_file(self):
        source = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')

        with open(source, 'rb') as handle:
            self.properties = pickle.load(handle)

        self.settings_import = True

    def choose_path(self, cont):
        sourceFile = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceFile

        if self.settings_import:
            self.dic_change[cont] = sourceFile

    def choose_dir(self, cont):
        sourceDir = filedialog.askdirectory(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceDir

        if self.settings_import:
            self.dic_change[cont] = sourceDir

    def callback(self, var, i):
        self.dic_change[i] = var.get()

    def submit(self, placeholders):

        if not self.settings_import:
            
            if self.east.get() in placeholders:
                self.east.set(-999)
            if self.west.get() in placeholders:
                self.west.set(-999)
            if self.north.get() in placeholders:
                self.north.set(-999)
            if self.south.get() in placeholders:
                self.south.set(-999)

            dic = {}
            dic['pred_path'] = self.dic_path['pred_path'] + '/prediction.nc'
            dic['geo_path'] = self.dic_path['geo_path']
            dic['feat_path'] = self.dic_path['feat_path']
            dic['east'] = float(self.east.get())
            dic['west'] = float(self.west.get())
            dic['north'] = float(self.north.get())
            dic['south'] = float(self.south.get())
            dic['from_scratch'] = self.from_scratch.get()
            dic['add'] = self.add.get()
            dic['delete'] = self.delete.get()
            dic['ohe'] = self.ohe.get()

            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_pred.pkl'

        elif self.settings_import:

            dic = {}

            dic['pred_path'] = self.properties['pred_path']
            dic['geo_path'] = self.properties['geo_path']
            dic['feat_path'] = self.properties['feat_path']
            dic['east'] = self.properties['east']
            dic['west'] = self.properties['west']
            dic['north'] = self.properties['north']
            dic['south'] = self.properties['south']
            dic['ohe'] = self.properties['ohe']
            dic['from_scratch'] = self.from_scratch.get()
            dic['add'] = self.add.get()
            dic['delete'] = self.delete.get()

            for key in self.dic_change:
                if key == 'pred_path':
                    dic[key] = self.dic_change[key] + '/prediction.nc'
                else:
                    if self.dic_change[key] not in placeholders:
                        dic[key] = self.dic_change[key]

        with open('tmp_pred.pkl', 'wb') as handle:
            pickle.dump(dic, handle)

        if self.save.get() == 1:

            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_pred.pkl'

            with open(sourceFile, 'wb') as handle:
                pickle.dump(dic, handle)

        self.master.destroy()
        if self.map:
            settings_map()


class settings_map:

    def __init__(self):

        def toggle_checkbox():
            if self.pred.get() == 1:
                b5.config(state="normal")
            else:
                b5.config(state="disabled")
                b5.deselect()

        self.master = tk.Tk()
        self.settings_import = False
        self.row = 0
        
        self.dic_path = {}
        self.dic_change = {}

        self.master.winfo_toplevel().title("Map generation - general settings")
        
        default_font = font.nametofont("TkDefaultFont")
        heading_font = font.Font(family=default_font.cget("family"), size=14, weight="bold")
        
        impath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')) + '/images/Logo.png'
        pil_image = Image.open(impath)
        resized_image = pil_image.resize((750, 200), Image.LANCZOS)
        image = ImageTk.PhotoImage(resized_image)
        
        #image = tk.PhotoImage(file=impath)
        image_label = tk.Label(self.master, image=image, width=750, height=200)
        image_label.image = image
        image_label.grid(row=self.row, column=0, columnspan=4)
        self.row = self.row + 1

        b_chooseFile = tk.Button(
            self.master, text="Import settings", command=self.import_file)
        b_chooseFile.grid(row=self.row, column=0, columnspan=4, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        Label(self.master, text='General settings', font=font.Font(family=default_font.cget("family"), weight='bold')).grid(row=self.row, column=0, sticky='w')
        self.row = self.row + 1
        
        Label(self.master, text="Path to training and prediction dataset", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.b_train = tk.Button(
            self.master,
            text="Choose path to training dataset",
            command=lambda: self.choose_path('train_path'))
        self.b_train.grid(row=self.row, column=1, columnspan=1, sticky='ew')

        self.b_pred = tk.Button(self.master, text="Choose path to prediction dataset",
                                command=lambda: self.choose_path('pred_path'))
        self.b_pred.grid(row=self.row, column=2, columnspan=2, sticky='ew')
        self.row = self.row + 1

        Label(self.master,
              text="Where do you want the\n models to be stored?", anchor='w', justify='left').grid(
                  row=self.row, column=0, sticky='w')
        self.b_model = tk.Button(self.master, text="Choose directory",
                                 command=lambda: self.choose_dir('model_path'))
        self.b_model.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="Foldername", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        
        self.model_to_load = tk.StringVar()
        i = 'model_to_load'
        self.model_to_load.trace_add(
            "write", lambda name, index, mode,
            var=self.model_to_load, i=i: self.callback(var, i))
        self.entry3 = tk.Entry(self.master, textvariable=self.model_to_load, fg='grey')
        placeholder3 = '...for loading'
        self.entry3.insert(0, placeholder3)  # Insert placeholder text initially
        self.entry3.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.grid(row=self.row, column=2, columnspan=2, sticky='ew')
        
        self.model_to_save = tk.StringVar()
        i = 'model_to_save'
        self.model_to_save.trace_add(
            "write", lambda name, index, mode,
            var=self.model_to_save, i=i: self.callback(var, i))
        self.entry4 = tk.Entry(self.master, textvariable=self.model_to_save, fg='grey')
        placeholder4 = '...for saving'
        self.entry4.insert(0, placeholder4)  # Insert placeholder text initially
        self.entry4.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.grid(row=self.row, column=1, sticky='ew')
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        Label(self.master, 
              text='Set-up of the training and prediction dataset', 
              font=font.Font(family=default_font.cget("family"), 
              weight='bold')).grid(row=self.row,
                                   column=0, 
                                   columnspan=2, 
                                   sticky='w')
        self.row = self.row + 1

        Label(self.master, text="Features not to consider", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        
        self.drop_train = tk.StringVar()
        i = 'drop_train'
        self.drop_train.trace_add(
            "write", lambda name, index, mode,
            var=self.drop_train, i=i: self.callback(var, i))
        self.entry1 = tk.Entry(self.master, textvariable=self.drop_train, fg='grey')
        placeholder1 = 'Drop from training dataset'
        self.entry1.insert(0, placeholder1)  # Insert placeholder text initially
        self.entry1.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.grid(row=self.row, column=1, sticky='ew')
        
        self.drop_pred = tk.StringVar()
        i = 'drop_pred'
        self.drop_pred.trace_add(
            "write", lambda name, index, mode,
            var=self.drop_pred, i=i: self.callback(var, i))
        self.entry2 = tk.Entry(self.master, textvariable=self.drop_pred, fg='grey')
        placeholder2 = 'Drop from prediction dataset'
        self.entry2.insert(0, placeholder2)  # Insert placeholder text initially
        self.entry2.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.grid(row=self.row, column=2, columnspan=2, sticky='ew')
        self.row = self.row + 1

        Label(self.master,
              text="Name of label column in training dataset", anchor='w', justify='left').grid(
                  row=self.row, column=0, sticky='w')
        self.name_label = tk.StringVar()
        i = 'name_label'
        self.name_label.trace_add(
            "write", lambda name, index, mode,
            var=self.name_label, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.name_label)
        entry.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1
    
        global all_buttons
        all_buttons = []
        
        Label(self.master, text="How to treat mismatching categories?", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        
        self.button_pressed = tk.StringVar()
        self.keep = tk.IntVar()
        self.keep.set(0)
        i = 'keep'
        self.keep.trace_add("write", lambda name, index, mode,
             var=self.keep, i=i: self.callback(var, i))
        self.b1 = tk.Radiobutton(self.master,
                         text="Keep instances and\n matching classes",
                         variable=self.keep,
                         value=1,
                         command=lambda: self.combined_command(self.b1, 'keep'),
                         anchor='w', justify='left')
        self.b1.grid(row=self.row, column=1, columnspan=1, sticky='w')

        
        self.remove_instances = tk.IntVar()
        self.remove_instances.set(0)
        i = 'remove_instances'
        self.remove_instances.trace_add("write", lambda name, index, mode,
             var=self.remove_instances, i=i: self.callback(var, i))
        self.b2 = tk.Radiobutton(self.master,
                         text="Remove instances of\n mismatching classes",
                         variable=self.remove_instances,
                         value=1,
                         command=lambda: self.combined_command(self.b2, 'remove'), anchor='w', justify='left')
        self.b2.grid(row=self.row, column=2, columnspan=1, sticky='w')
        
        all_buttons.append(self.b1)
        all_buttons.append(self.b2)
        
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        Label(self.master, text='Random Forest training and prediction', font=font.Font(family=default_font.cget("family"), weight='bold')).grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1

        Label(self.master, text="Number of trees:").grid(row=self.row, column=0, sticky='w')
        self.num_trees = tk.IntVar()
        i = 'num_trees'
        self.num_trees.trace_add(
            "write", lambda name, index, mode,
            var=self.num_trees, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.num_trees)
        entry.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1

        Label(self.master, text="Depth of the trees:", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.depth_trees = tk.IntVar()
        i = 'depth_trees'
        self.depth_trees.trace_add(
            "write", lambda name, index, mode,
            var=self.depth_trees, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.depth_trees)
        entry.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1

        Label(self.master, text="Evaluation criterion:", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.criterion = tk.StringVar()
        i = 'criterion'
        self.criterion.trace_add(
            "write", lambda name, index, mode,
            var=self.criterion, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.criterion)
        entry.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1

        Label(self.master, text="Size of validation dataset (0...1):", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.size_val = tk.DoubleVar()
        i = 'size_val'
        self.size_val.trace_add(
            "write", lambda name, index, mode,
            var=self.size_val, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.size_val)
        entry.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        self.save = tk.IntVar()
        b4 = Checkbutton(self.master,
                         text="Save above settings for later use",
                         variable=self.save, anchor='w',
                         font=font.Font(family=default_font.cget("family"), weight="bold"))
        b4.grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="What do you want to do?", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.train = tk.IntVar()
        b1 = Checkbutton(self.master,
                         text="Model training",
                         variable=self.train, anchor='w')
        b1.grid(row=self.row, column=1, sticky='w') 
        
        self.pred = tk.IntVar()
        b2 = Checkbutton(self.master,
                         text="Mapping",
                         variable=self.pred,
                         command=toggle_checkbox, anchor='w')
        b2.grid(row=self.row, column=2, sticky='w')

        self.yes = tk.IntVar()
        b5 = Checkbutton(self.master,
                         text="Predict in parallel",
                         variable=self.yes,
                         state="disabled", anchor='w')
        b5.grid(row=self.row, column=3, sticky='w')
        self.row = self.row + 1
        
        all_placeholders = []
        all_placeholders.append(placeholder1)
        all_placeholders.append(placeholder2)
        all_placeholders.append(placeholder3)
        all_placeholders.append(placeholder4)

        btn = tk.Button(self.master, text="Submit", command=lambda: self.submit(all_placeholders))
        btn.grid(row=self.row, column=0, columnspan=4, sticky='ew')
        
    def combined_command(self, selected_button, option):
        
        self.disable_other_buttons(selected_button)
        self.button_pressed = option
        
    def disable_other_buttons(self, selected_button):
        for button in all_buttons:
            if button != selected_button:
                button.config(state=tk.DISABLED)
        
    def on_entry_click(self, event, entry, placeholder):
        """Function that handles the event when the entry field is clicked."""
        if entry.get() == placeholder:
            entry.delete(0, tk.END)  # Delete all the text in the entry
            entry.config(fg='black')  # Change the text color to black
    
    def on_focus_out(self, event, entry, placeholder):
        """Function that handles the event when the entry field loses focus."""
        if entry.get() == '':
            entry.insert(0, placeholder)  # Insert the placeholder text
            entry.config(fg='grey')  # Change the text color to grey

    def callback(self, var, i):
        self.dic_change[i] = var.get()

    def choose_path(self, cont):
        sourceFile = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceFile

        if self.settings_import:
            self.dic_change[cont] = sourceFile

    def choose_dir(self, cont):
        sourceDir = filedialog.askdirectory(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceDir

        if self.settings_import:
            self.dic_change[cont] = sourceDir

    def import_file(self):
        self.settings_import = True
        print(self.settings_import)
        source = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')

        with open(source, 'rb') as handle:
            self.properties = pickle.load(handle)

    def submit(self, placeholders):

        if not self.settings_import:
            
            if self.drop_train in placeholders:
                self.drop_train.set('')
            if self.pred_train in placeholders:
                self.drop_pred.set('')
            if self.model_to_load in placeholders:
                self.model_to_load.set('')
            if self.model_to_save in placeholders:
                self.model_to_save.set('')

            dic = {}
            dic['train_path'] = self.dic_path['train_path']
            dic['pred_path'] = self.dic_path['pred_path']
            dic['model_path'] = self.dic_path['model_path']
            dic['drop_train'] = self.drop_train.get()
            dic['drop_pred'] = self.drop_pred.get()
            dic['model_to_load'] = self.model_to_load.get()
            dic['model_to_save'] = self.model_to_save.get()
            dic['num_trees'] = self.num_trees.get()
            dic['size_val'] = self.size_val.get()
            dic['depth_trees'] = self.depth_trees.get()
            dic['name_label'] = self.name_label.get()
            dic['criterion'] = self.criterion.get()
            dic['keep'] = self.keep.get()
            dic['remove_instances'] = self.remove_instances.get()

            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_map.pkl'

        elif self.settings_import:
        

            dic = {}
            dic['train_path'] = self.properties['train_path']
            dic['pred_path'] = self.properties['pred_path']
            dic['model_path'] = self.properties['model_path']
            dic['drop_train'] = self.properties['drop_train']
            dic['drop_pred'] = self.properties['drop_pred']
            dic['model_to_load'] = self.properties['model_to_load']
            dic['model_to_save'] = self.properties['model_to_save']
            dic['num_trees'] = self.properties['num_trees']
            dic['size_val'] = self.properties['size_val']
            dic['depth_trees'] = self.properties['depth_trees']
            dic['name_label'] = self.properties['name_label']
            dic['criterion'] = self.properties['criterion']
            dic['keep'] = self.properties['keep']
            dic['remove_instances'] = self.properties['remove_instances']

            for key in self.dic_change:
                if self.dic_change[key] not in placeholders:
                    dic[key] = self.dic_change[key]

        if self.save.get() == 1:
            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_map.pkl'

            with open(sourceFile, 'wb') as handle:
                pickle.dump(dic, handle)

        if self.yes == 0:
            dic['training'] = self.train.get()
            dic['prediction'] = self.pred.get()
            dic['parallel'] = self.yes
        else:
            dic['training'] = self.train.get()
            dic['prediction'] = self.pred.get()
            dic['parallel'] = self.yes.get()
        with open('tmp_map.pkl', 'wb') as handle:
            pickle.dump(dic, handle)

        self.master.destroy()


class settings_train:

    def __init__(self, pred, maps):
        super().__init__()
        self.pred = pred
        self.map = maps
        self.master = tk.Tk()
        self.row = 0

        self.master.winfo_toplevel().title(
            "Training dataset generation - general settings")
        
        default_font = font.nametofont("TkDefaultFont")
        heading_font = font.Font(family=default_font.cget("family"), size=14, weight="bold")
        
        global all_buttons
        all_buttons = []
        
        impath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')) + '/images/Logo.png'
        pil_image = Image.open(impath)
        resized_image = pil_image.resize((600, 150), Image.LANCZOS)
        image = ImageTk.PhotoImage(resized_image)
        
        #image = tk.PhotoImage(file=impath)
        image_label = tk.Label(self.master, image=image, width=600, height=150)
        image_label.image = image
        image_label.grid(row=self.row, column=0, columnspan=4)
        self.row = self.row + 1
        
        self.settings_import = False
        b_chooseFile = tk.Button(self.master,
                                 text="Import settings",
                                 command=self.import_file)
        b_chooseFile.grid(row=self.row, column=0, columnspan=4, sticky='ew')

        self.dic_path = {}
        self.dic_change = {}
        
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1
        
        Label(self.master, text="General settings", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1

        Label(self.master, text="Provide path to:", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.b_ls = tk.Button(self.master,
                              text="Landslide data",
                              command=lambda: self.choose_path('ls_path'))
        self.b_ls.grid(row=self.row, column=1, columnspan=1, sticky='ew')

        self.b_gs = tk.Button(self.master,
                              text="dataset_summary.csv",
                              command=lambda: self.choose_path('geo_path'))
        self.b_gs.grid(row=self.row, column=2, columnspan=1, sticky='ew')

        self.b_nonls = tk.Button(
            self.master,
            text="Absence locations",
            command=lambda: self.choose_path('nonls_path'))
        self.b_nonls.grid(row=self.row, column=3, columnspan=1, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="Features to include", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.b_feat = tk.Button(self.master,
                                text="Path to keys_to_include.csv",
                                command=lambda: self.choose_path('feat_path'))
        self.b_feat.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1

        Label(self.master,
              text="Path to directory for storing\n the training dataset", anchor='w', justify='left').grid(
                  row=self.row, column=0, sticky='w')
        self.b_train = tk.Button(self.master,
                                 text="Choose directory",
                                 command=lambda: self.choose_dir('train_path'))
        self.b_train.grid(row=self.row, column=1, columnspan=3, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="Number of absence locations\n in the training dataset:", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        self.num_nonls = tk.IntVar()
        i = 'num_nonls'
        self.num_nonls.trace_add(
            "write", lambda name, index, mode,
            var=self.num_nonls, i=i: self.callback(var, i))
        entry = tk.Entry(self.master, textvariable=self.num_nonls)
        entry.grid(row=self.row, column=1, columnspan=1, sticky='ew')
        
        self.ohe = tk.IntVar()
        b11 = Checkbutton(self.master, text="One-hot encoding of categorical variables",
                         variable=self.ohe)#, command=lambda: button_pressed.set('cluster'))
        b11.grid(row=self.row, column=2, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="Information on the landslide and absence locations inventory", anchor='w', justify='left', font=heading_font).grid(
            row=self.row, column=0, columnspan=3, sticky='w')
        self.row = self.row + 1
        Label(self.master, text="Column name in landslide\n inventory containing...", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.x = tk.StringVar()
        i = 'x'
        self.x.trace_add(
            "write", lambda name, index, mode,
            var=self.x, i=i: self.callback(var, i))
        self.entry3 = tk.Entry(self.master, textvariable=self.x, fg='grey')
        placeholder3 = 'Longitude values'
        self.entry3.insert(0, placeholder3)  # Insert placeholder text initially
        self.entry3.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry3, placeholder=placeholder3))
        self.entry3.grid(row=self.row, column=1, sticky='ew')

        self.y = tk.StringVar()
        i = 'y'
        self.y.trace_add(
            "write", lambda name, index, mode,
            var=self.y, i=i: self.callback(var, i))
        self.entry4 = tk.Entry(self.master, textvariable=self.y, fg='grey')
        placeholder4 = 'Latitude values'
        self.entry4.insert(0, placeholder4)  # Insert placeholder text initially
        self.entry4.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry4, placeholder=placeholder4))
        self.entry4.grid(row=self.row, column=2, sticky='ew')
        
        #Label(self.master, text="Column name of the landslide ID", anchor='w', justify='left').grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.id = tk.StringVar()
        i = 'id'
        self.id.trace_add(
            "write", lambda name, index, mode,
            var=self.id, i=i: self.callback(var, i))
        self.entry5 = tk.Entry(self.master, textvariable=self.id, fg='grey')
        placeholder5 = 'Landslide ID'
        self.entry5.insert(0, placeholder5)  # Insert placeholder text initially
        self.entry5.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry5, placeholder=placeholder5))
        self.entry5.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry5, placeholder=placeholder5))
        self.entry5.grid(row=self.row, column=3, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master, text="Variable in absence\n locations inventory containing...", anchor='w', justify='left').grid(row=self.row, column=0, sticky='w')
        self.x_nonls = tk.StringVar()
        i = 'x_nonls'
        self.x_nonls.trace_add(
            "write", lambda name, index, mode,
            var=self.x_nonls, i=i: self.callback(var, i))
        self.entry1 = tk.Entry(self.master, textvariable=self.x_nonls, fg='grey')
        placeholder1 = 'Longitude values'
        self.entry1.insert(0, placeholder1)  # Insert placeholder text initially
        self.entry1.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry1, placeholder=placeholder1))
        self.entry1.grid(row=self.row, column=1, sticky='ew')

        self.y_nonls = tk.StringVar()
        i = 'y_nonls'
        self.y_nonls.trace_add(
            "write", lambda name, index, mode,
            var=self.y_nonls, i=i: self.callback(var, i))
        self.entry2 = tk.Entry(self.master, textvariable=self.y_nonls, fg='grey')
        placeholder2 = 'Latitude values'
        self.entry2.insert(0, placeholder2)  # Insert placeholder text initially
        self.entry2.bind('<FocusIn>', lambda event:self.on_entry_click(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.bind('<FocusOut>', lambda event:self.on_focus_out(event, entry=self.entry2, placeholder=placeholder2))
        self.entry2.grid(row=self.row, column=2, sticky='ew')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        self.save = tk.IntVar()
        b4 = Checkbutton(self.master,
                         text="Save above settings for later use",
                         variable=self.save, font=font.Font(family=default_font.cget("family"), weight="bold"))
        b4.grid(row=self.row, column=0, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        Label(self.master).grid(row=self.row, column=0)
        self.row = self.row + 1

        Label(self.master, text="Choose from:", anchor='w', justify='left').grid(
            row=self.row, column=0, sticky='w')
        button_pressed = StringVar()
        self.from_scratch = tk.IntVar()
        self.from_scratch.set(0)
        b1 = tk.Radiobutton(self.master,
                            text="Generate training dataset from scratch",
                            variable=button_pressed,
                            value='from_scratch',
                            command=lambda: self.disable_other_buttons(b1))
        b1.grid(row=self.row, column=1, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        self.delete = tk.IntVar()
        self.delete.set(0)
        b2 = tk.Radiobutton(self.master,
                            text="Delete feature(s) from existing training dataset",
                            variable=button_pressed,
                            value='delete',
                            command=lambda: self.disable_other_buttons(b2))
        b2.grid(row=self.row, column=1, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        self.add = tk.IntVar()
        self.add.set(0)
        b3 = tk.Radiobutton(self.master,
                            text="Add feature(s) from existing training dataset",
                            variable=button_pressed,
                            value='add',
                            command=lambda: self.disable_other_buttons(b3))
        b3.grid(row=self.row, column=1, columnspan=2, sticky='w')
        self.row = self.row + 1
        
        all_buttons.append(b1)
        all_buttons.append(b2)
        all_buttons.append(b3)
        
        all_placeholders = []
        all_placeholders.append(placeholder1)
        all_placeholders.append(placeholder2)
        all_placeholders.append(placeholder3)
        all_placeholders.append(placeholder4)
        all_placeholders.append(placeholder5)
        
        b2.wait_variable(button_pressed)

        if button_pressed.get() == 'from_scratch':

            Label(self.master,
                  text="Compilation using:", anchor='w', justify='left').grid(
                      row=self.row, column=0, sticky='w')
            
            self.cluster = tk.IntVar()
            self.cluster.set(0)
            b11 = tk.Radiobutton(self.master, text="Clustering",
                         variable=self.cluster,
                         value=1,
                         command= lambda: self.disable_other_buttons(b11))
            b11.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            self.interpolation = tk.IntVar()
            self.interpolation.set(0)
            b22 = tk.Radiobutton(
                self.master,
                text="Interpolation",
                variable=self.interpolation,
                value=1,
                command= lambda: self.disable_other_buttons(b22))
            b22.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            self.no_interpolation = tk.IntVar()
            self.no_interpolation.set(0)
            b33 = tk.Radiobutton(
                self.master,
                text="No interpolation",
                variable=self.no_interpolation,
                value=1,
                command= lambda: self.disable_other_buttons(b33))
            b33.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            all_buttons = []
            all_buttons.append(b11)
            all_buttons.append(b22)
            all_buttons.append(b33)

            self.add.set(0)
            self.delete.set(0)
            self.from_scratch.set(1)
            if self.cluster.get() == 1:
                self.interpolation.set(0)
                self.no_interpolation.set(0)
            elif self.interpolation.get() == 1:
                self.cluster.set(0)
                self.no_interpolation.set(0)
            if self.no_interpolation.get() == 1:
                self.cluster.set(0)
                self.interpolation.set(0)

            btn = tk.Button(self.master, text="Submit", command=lambda: self.submit(all_placeholders))
            btn.grid(row=self.row, column=0, columnspan=4, sticky='ew')
            
        elif button_pressed.get() == 'add':
            
            Label(self.master,
                  text="Compilation using:", anchor='w', justify='left').grid(
                      row=self.row, column=0, sticky='w')
            
            self.cluster = tk.IntVar()
            self.cluster.set(0)

            b111 = tk.Radiobutton(self.master, text="Clustering",
                         variable=self.cluster,
                         value=1,
                         command= lambda: self.disable_other_buttons(b111))
            b111.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            self.interpolation = tk.IntVar()
            self.interpolation.set(0)
            b222 = tk.Radiobutton(
                self.master,
                text="Interpolation",
                variable=self.interpolation,
                value=1,
                command= lambda: self.disable_other_buttons(b222))
            b222.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            self.no_interpolation = tk.IntVar()
            self.no_interpolation.set(0)
            b333 = tk.Radiobutton(
                self.master,
                text="No interpolation",
                variable=self.no_interpolation,
                value=1,
                command= lambda: self.disable_other_buttons(b333))
            b333.grid(row=self.row, column=1, sticky='w')
            self.row = self.row + 1
            
            all_buttons = []
            all_buttons.append(b111)
            all_buttons.append(b222)
            all_buttons.append(b333)

            self.add.set(1)
            self.delete.set(0)
            self.from_scratch.set(0)
                
            btn = tk.Button(self.master, text="Submit", command=lambda: self.submit(all_placeholders))
            btn.grid(row=self.row, column=0, columnspan=4, sticky='ew')
            
        elif button_pressed.get() == 'delete':
            self.add.set(0)
            self.delete.set(1)
            self.from_scratch.set(0)
            self.cluster = tk.IntVar()
            self.cluster.set(0)
            self.no_interpolation = tk.IntVar()
            self.no_interpolation.set(0)
            self.interpolation = tk.IntVar()
            self.interpolation.set(0)
            
            btn = tk.Button(self.master, text="Submit", command=lambda: self.submit(all_placeholders))
            btn.grid(row=self.row, column=0, columnspan=4, sticky='ew')
            
    def disable_other_buttons(self, selected_button):
            for button in all_buttons:
                if button != selected_button:
                    button.config(state=tk.DISABLED)
                    
    def on_entry_click(self, event, entry, placeholder):
        """Function that handles the event when the entry field is clicked."""
        if entry.get() == placeholder:
            entry.delete(0, tk.END)  # Delete all the text in the entry
            entry.config(fg='black')  # Change the text color to black
    
    def on_focus_out(self, event, entry, placeholder):
        """Function that handles the event when the entry field loses focus."""
        if entry.get() == '':
            entry.insert(0, placeholder)  # Insert the placeholder text
            entry.config(fg='grey')  # Change the text color to grey

    def callback(self, var, i):
        self.dic_change[i] = var.get()

    def choose_path(self, cont):
        sourceFile = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceFile

        if self.settings_import:
            self.dic_change[cont] = sourceFile

    def choose_dir(self, cont):
        sourceDir = filedialog.askdirectory(
            parent=self.master, initialdir="/", title='Choose path')
        self.dic_path[cont] = sourceDir

        if self.settings_import:
            if cont == 'train_path':
                self.dic_change[cont] = sourceDir + '/training.csv'
            else:
                self.dic_change[cont] = sourceDir

    def import_file(self):
        self.settings_import = True
        print(self.settings_import)
        source = filedialog.askopenfilename(
            parent=self.master, initialdir="/", title='Choose path')

        with open(source, 'rb') as handle:
            self.properties = pickle.load(handle)

    def submit(self, placeholders):

        if not self.settings_import:
            
            if self.x.get() in placeholders:
                self.x.set('')
            if self.y.get() in placeholders:
                self.y.set('')
            if self.x_nonls.get() in placeholders:
                self.x_nonls.set('')
            if self.y_nonls.get() in placeholders:
                self.y_nonls.set('')
            if self.id.get() in placeholders:
                self.id.set('')

            dic = {}
            dic['ls_path'] = self.dic_path['ls_path']
            dic['nonls_path'] = self.dic_path['nonls_path']
            dic['train_path'] = self.dic_path['train_path'] + '/training.csv'
            dic['geo_path'] = self.dic_path['geo_path']
            dic['feat_path'] = self.dic_path['feat_path']
            dic['x'] = self.x.get()
            dic['y'] = self.y.get()
            dic['id'] = self.id.get()
            dic['x_nonls'] = self.x_nonls.get()
            dic['y_nonls'] = self.y_nonls.get()
            dic['num_nonls'] = int(self.num_nonls.get())
            dic['from_scratch'] = self.from_scratch.get()
            dic['delete'] = self.delete.get()
            dic['add'] = self.add.get()
            dic['cluster'] = self.cluster.get()
            dic['interpolation'] = self.interpolation.get()
            dic['no_interpolation'] = self.no_interpolation.get()
            dic['ohe'] = self.ohe.get()

            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_train.pkl'

        elif self.settings_import:

            dic = {}
            dic['ls_path'] = self.properties['ls_path']
            dic['nonls_path'] = self.properties['nonls_path']
            dic['train_path'] = self.properties['train_path']
            dic['geo_path'] = self.properties['geo_path']
            dic['feat_path'] = self.properties['feat_path']
            dic['x'] = self.properties['x']
            dic['y'] = self.properties['y']
            dic['id'] = self.properties['id']
            dic['x_nonls'] = self.properties['x_nonls']
            dic['y_nonls'] = self.properties['y_nonls']
            dic['num_nonls'] = self.properties['num_nonls']
            dic['from_scratch'] = self.from_scratch.get()
            dic['delete'] = self.delete.get()
            dic['add'] = self.add.get()
            dic['cluster'] = self.cluster.get()
            dic['no_interpolation'] = self.no_interpolation.get()
            dic['interpolation'] = self.interpolation.get()
            dic['ohe'] = self.properties['ohe']

            for key in self.dic_change:
                if self.dic_change[key] not in placeholders:
                    dic[key] = self.dic_change[key]

        with open('tmp_train.pkl', 'wb') as handle:
            pickle.dump(dic, handle)

        if self.save.get() == 1:
            sourceDir = filedialog.askdirectory(
                parent=self.master, initialdir="/", title='Choose path')
            sourceFile = sourceDir + '/settings_train.pkl'

            with open(sourceFile, 'wb') as handle:
                pickle.dump(dic, handle)

        self.master.destroy()
        if self.pred:
            settings_pred(self.map)
        elif not self.pred and self.map:
            settings_map()
